@@ -1,5 +1,7 @@
 from collections import OrderedDict
 from . import content_encoding, content_type
+from lyrebird.utils import CaseInsensitiveDict
+import json
 
 origin2flow_handlers = OrderedDict({
     'Content-Encoding': content_encoding,
@@ -23,8 +25,15 @@ class DataHelper:
         if not _data:
             return
 
+        # Read raw headers, support the request from extra mock 9999 port
+        if 'Proxy-Raw-Headers' in origin_obj.headers:
+            _origin_headers = json.loads(origin_obj.headers['Proxy-Raw-Headers'])
+            raw_headers = CaseInsensitiveDict(_origin_headers)
+        else:
+            raw_headers = origin_obj.headers
+
         for headers_key, func in origin2flow_handlers.items():
-            headers_val = origin_obj.headers.get(headers_key, '')
+            headers_val = raw_headers.get(headers_key, '')
             _data = func.origin2flow(headers_val, _data)
 
         if output:
