@@ -76,7 +76,7 @@ class LyrebirdMockServer(ThreadServer):
 
     def run(self):
         server_ip = application.config.get('ip')
-        _logger.warning(f'start on http://{server_ip}:{self.port}')
+        _logger.warning(f'Core start on http://{server_ip}:{self.port}')
         self.socket_io.run(self.app, host='0.0.0.0', port=self.port, debug=self.debug, use_reloader=False)
 
     def stop(self):
@@ -89,4 +89,4 @@ class LyrebirdMockServer(ThreadServer):
             self.socket_io.stop()
         except Exception:
             pass
-        _logger.warning('MockServer shutdown')
+        _logger.warning('CoreServer shutdown')
