@@ -1,7 +1,11 @@
 import re
+import os
 import math
 import time
 import socket
+import tarfile
+import requests
+from pathlib import Path
 from contextlib import closing
 
 
@@ -62,6 +66,42 @@ def find_free_port():
         return s.getsockname()[1]
 
 
+def compress_tar(input_path, output_path, suffix=None):
+    current_path = Path.cwd()
+    input_path = Path(input_path).expanduser().absolute().resolve()
+    output_path = Path(output_path).expanduser().absolute().resolve()
+    output_file = f'{output_path}{suffix}' if suffix else output_path
+
+    os.chdir(input_path)
+    # If not chdir, the directory in the compressed file will start from the root directory
+    tar = tarfile.open(output_file, 'w:gz')
+    for root, dirs, files in os.walk(input_path):
+        for f in files:
+            tar.add(f, recursive=False)
+    tar.close()
+    os.chdir(current_path)
+    return output_file
+
+
+def decompress_tar(input_path, output_path=None):
+    input_path = Path(input_path).expanduser().absolute().resolve()
+    if not output_path:
+        filename = input_path.stem if input_path.suffix else f'{input_path.name}-decompress'
+        output_path = input_path.parent / filename
+
+    tf = tarfile.open(str(input_path))
+    tf.extractall(str(output_path))
+    tf.close()
+    return output_path
+
+
+def download(link, input_path):
+    resp = requests.get(link, stream=True)
+    with open(input_path, 'wb') as f:
+        for chunck in resp.iter_content():
+            f.write(chunck)
+
+
 class CaseInsensitiveDict(dict):
 
     def __init__(self, dict):
